unit Unit1;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Forms, Controls, Graphics, Dialogs, ExtCtrls, StdCtrls,
  Spin, LvlGraphCtrl, IpHtml, IpHtmlNodes;

type

  TClassDisplay = (cdNone, cdNodes);

  { TForm1 }

  TForm1 = class(TForm)
    btnNodeHierarchy: TButton;
    cbStraightenGraph: TCheckBox;
    cbReduceBackEdges: TCheckBox;
    cbHighLevels: TCheckBox;
    Label2: TLabel;
    Label3: TLabel;
    seCaptionScale: TFloatSpinEdit;
    GroupBox1: TGroupBox;
    ImageList1: TImageList;
    Label1: TLabel;
    LvlGraphControl1: TLvlGraphControl;
    Panel1: TPanel;
    seGapBottom: TSpinEdit;
    seGapTop: TSpinEdit;
    procedure btnNodeHierarchyClick(Sender: TObject);
    procedure cbStraightenGraphChange(Sender: TObject);
    procedure cbReduceBackEdgesChange(Sender: TObject);
    procedure cbHighLevelsChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure seCaptionScaleChange(Sender: TObject);
    procedure seGapChange(Sender: TObject);
  private
    FDisplay: TClassDisplay;
    FRootClassName: String;
    function AddClassNode(AClass: TClass; ARootClassName: String = ''): TLvlGraphNode;
    function GetClassImageIndex(AClassName: String): Integer;
    procedure DefaultLvlGraphSettings;
    procedure RecreateGraph;

  public

  end;

var
  Form1: TForm1;

implementation

{$R *.lfm}

const
  // Image indices
  IDX_COMPONENT = 0;


{ TForm1 }

procedure TForm1.FormCreate(Sender: TObject);
begin
  LvlGraphControl1.Images := ImageList1;
  LvlGraphControl1.NodeStyle.CaptionScale := seCaptionScale.Value;
  LvlGraphControl1.NodeStyle.GapTop := seGapTop.Value;
  LvlGraphControl1.NodeStyle.GapBottom := seGapBottom.Value;
end;

procedure TForm1.cbReduceBackEdgesChange(Sender: TObject);
var
  i: Integer;
begin
  LvlGraphControl1.Clear;

  if cbReduceBackEdges.Checked then
    LvlGraphControl1.Options := LvlGraphControl1.Options + [lgoReduceBackEdges]
  else
    LvlGraphControl1.Options := LvlGraphControl1.Options - [lgoReduceBackEdges];

  RecreateGraph;
end;

procedure TForm1.cbHighLevelsChange(Sender: TObject);
var
  i: Integer;
begin
  LvlGraphcontrol1.Clear;

  if cbHighLevels.Checked then
    LvlGraphControl1.Options := LvlGraphControl1.Options + [lgoHighLevels]
  else
    LvlGraphControl1.Options := LvlGraphControl1.Options - [lgoHighLevels];

  RecreateGraph;
end;

procedure TForm1.cbStraightenGraphChange(Sender: TObject);
var
  i: Integer;
begin
  LvlGraphControl1.Clear;

  if cbStraightenGraph.Checked then
    LvlGraphControl1.Options := LvlGraphControl1.Options + [lgoStraightenGraph]
  else
    LvlGraphControl1.Options := LvlGraphControl1.Options - [lgoStraightenGraph];

  RecreateGraph;
end;

function TForm1.GetClassImageIndex(AClassName: String): Integer;
begin
  {
  case AClassName of
    else
    }
      Result := IDX_COMPONENT;
//  end;
end;

function TForm1.AddClassNode(AClass: TClass; ARootClassName: String = ''): TLvlGraphNode;
var
  parentclass: TClass;
begin
  if ARootClassName <> '' then
    FRootClassName := ARootClassName;
  Result := LvlGraphControl1.Graph.GetNode(AClass.ClassName, true);
  Result.ImageIndex := GetClassImageIndex(AClass.ClassName);
  if AClass.ClassName = FRootClassName then
    exit;
  parentclass := AClass.ClassParent;
  AddClassNode(parentclass);
  LvlGraphControl1.Graph.GetEdge(parentclass.ClassName, AClass.ClassName, true);
end;

procedure TForm1.btnNodeHierarchyClick(Sender: TObject);
begin
  FDisplay := cdNodes;

  LvlGraphControl1.Clear;
  AddClassNode(TIpHtmlNode, 'TPersistent');
  AddClassNode(TIpHtmlNodeText);
  AddClassNode(TIpHtmlNodeNv);
    AddClassNode(TIpHtmlNodeMETA);
    AddClassNode(TIpHtmlNodePARAM);
    AddClassNode(TIpHtmlNodeSCRIPT);
    AddClassNode(TIpHtmlNodeTitle);
  AddClassNode(TIpHtmlNodeMulti);
    AddClassNode(TIpHtmlNodeCore);
      AddClassNode(TIpHtmlNodeBlock);
        AddClassNode(TIpHtmlNodeBODY);
        AddClassNode(TIpHtmlNodeCAPTION);
        AddClassNode(TIpHtmlNodeTableHeaderOrCell);
          AddClassNode(TIpHtmlNodeTH);
          AddClassNode(TIpHtmlNodeTD);
      AddClassNode(TIpHtmlNodeInline);
        AddClassNode(TIpHtmlNodeA);
        AddClassNode(TIpHtmlNodeADDRESS);
        AddClassNode(TIpHtmlNodeAPPLET);
        AddClassNode(TIpHtmlNodeAlignInline);
          AddClassNode(TIpHtmlNodeControl);
            AddClassNode(TIpHtmlNodeBUTTON);
            AddClassNode(TIpHtmlNodeIFRAME);
            AddClassNode(TIpHtmlNodeINPUT);
            AddClassNode(TIpHtmlNodeSELECT);
            AddclassNode(TIpHtmlNodeTEXTAREA);
          AddClassNode(TIpHtmlNodeHR);
          AddClassNode(TIpHtmlNodeIMG);
          AddClassNode(TIpHtmlNodeLI);
          AddClassNode(TIpHtmlNodeTABLE);
        AddClassNode(TIpHtmlNodeBLINK);
        AddClassNode(TIpHtmlNodeBLOCKQUOTE);
        AddClassNode(TIpHtmlNodeBR);
        AddClassNode(TIpHtmlNodeDD);
        AddClassNode(TIpHtmlNodeDIV);
        AddClassNode(TIpHtmlNodeDL);
        AddClassNode(TIpHtmlNodeDT);
        AddClassNode(TIpHtmlNodeFORM);
        AddClassNode(TIpHtmlNodeGenInline);
          AddClassNode(TIpHtmlNodeBASEFONT);
          AddClassNode(TIpHtmlNodeDEL);
          AddClassNode(TIpHtmlNodeFONT);
          AddClassNode(TIpHtmlNodeFontStyle);
          AddClassNode(TIpHtmlNodeINS);
          AddClassNode(TIpHtmlNodeNOBR);
          AddClassNode(TIpHtmlNodePhrase);
          AddClassNode(TIpHtmlNodeSPAN);
        AddClassNode(TIpHtmlNodeOL);
        AddClassNode(TIpHtmlNodeHeader);
        AddClassNode(TIpHtmlNodeLABEL);
        AddClassNode(TIpHtmlNodeList);
          AddclassNode(TIpHtmlNodeUL);
          AddClassNode(TIpHtmlNodeDIR);
          AddClassNode(TIpHtmlNodeMENU);
        AddClassNode(TIpHtmlNodeNOSCRIPT);
        AddClassNode(TIpHtmlNodeOBJECT);
        AddClassNode(TIpHtmlNodeP);
        AddClassNode(TIpHtmlNodePRE);
        AddClassNode(TIpHtmlNodeQ);
      AddClassNode(TIpHtmlNodeFRAMESET);
      AddClassNode(TIpHtmlNodeAREA);
      AddClassNode(TIpHtmlNodeCOL);
      AddClassNode(TIpHtmlNodeCOLGROUP);
      AddClassNode(TIpHtmlNodeFIELDSET);
      AddClassNode(TIpHtmlNodeFRAME);
      AddClassNode(TIpHtmlNodeLEGEND);
      AddClassNode(TIpHtmlNodeLINK);
      AddClassNode(TIpHtmlNodeMAP);
      AddClassNode(TIpHtmlNodeNOFRAMES);
      AddClassNode(TIpHtmlNodeOPTGROUP);
      AddClassNode(TIpHtmlNodeOPTION);
      AddClassNode(TIpHtmlNodeTHeadFootBody);
        AddClassNode(TIpHtmlNodeTBODY);
        AddClassNode(TIpHtmlNodeTFOOT);
        AddClassNode(TIpHtmlNodeTHEAD);
      AddClassNode(TIpHtmlNodeTR);
    AddClassNode(TIpHtmlNodeHEAD);
    AddClassNode(TIpHtmlNodeSTYLE);
    AddClassNode(TIpHtmlNodeHTML);

  DefaultLvlGraphSettings;
end;

procedure TForm1.DefaultLvlGraphSettings;
begin
  with LvlGraphControl1 do begin
    NodeStyle.Shape := lgnsNone;
    NodeStyle.Width := 15;
    NodeStyle.GapTop := seGapTop.Value;  //12;
    NodeStyle.GapBottom := seGapBottom.Value;  // 0;
    NodeStyle.CaptionPosition := lgncBottom;
    NodeStyle.CaptionScale := seCaptionScale.Value;  //1.0;

    EdgeStyle.Shape := lgesStraight;

    Limits.MaxLevelHeightAbs := MaxInt;

    Font.Size := 10;
  end;
end;

procedure TForm1.RecreateGraph;
begin
  case FDisplay of
    cdNone: ;
    cdNodes: btnNodeHierarchyClick(nil);
  end;
end;

procedure TForm1.seCaptionScaleChange(Sender: TObject);
begin
  LvlGraphControl1.NodeStyle.CaptionScale := seCaptionScale.Value;
end;

procedure TForm1.seGapChange(Sender: TObject);
begin
  LvlGraphControl1.Clear;

  if Sender = seGapTop then
    LvlGraphControl1.NodeStyle.GapTop := seGapTop.Value;
  if Sender = seGapBottom then
    LvlGraphControl1.NodeStyle.GapBottom := seGapBottom.Value;

  RecreateGraph;
end;


end.

